
JONSWAP<-function(w,hs,tp,gam=NULL){
  #f(w)
  #Parameters
  #tp= 1/wp wp frequencia dominante
  #hs heigh significance (expected value of the third highest waves)
  sa<-0.07
  sb<-0.09
  s<-ifelse(w<(1/tp),sa,sb)
  if(is.null(gam))gam<-exp(3.484*(1-0.1975*(0.036-0.0056*tp/sqrt(hs))*tp^4/hs^2))
  g<-9.8
  #bj<-((0.0624)/(.23+.033*gam-0.185*(1/(1.9+gam))))*(1.094-0.1915*log(gam))
  sw<- (g^2)*(w^(-5))*exp(-(5/4)*(1/(tp*w)^4))*(gam^(exp(-(tp*w-1)^2/(2*s^2))))
  return(sw)
}
G<-function(theta,w,theta0,tp,smax=10,U=NULL){
  #G(theta,w)
  #Parameters
  #U wind speed (think like the mean wind speed)
  #tp= 1/wp wp frequencia dominante
  #smax if U is not specified
  ## Empirical: smax=10, wind-waves, smax=25, swell with short decay
  #             smax=75, swell with long decay
  #theta0 main direction
  if(is.null(U)==FALSE){
    g<-9.8
    smax<-11.5*((1/tp)*U/g)^(-2.5)
  }
  s<-ifelse(w<(1/tp),((w*tp)^5)*smax,((w*tp)^(-2.5))*smax)
  G0<-(2^(2*s-1))*((gamma(s+1))^2)/(gamma(2*s+1))
  return(G0*(cos((theta-theta0)/2))^(2*s))
}

check.S.nan<-function(S.w.theta,theta0){
  col.with.nan<-which(is.nan(S.w.theta)==TRUE,arr.ind = TRUE)
  if(length(col.with.nan)>0){
    theta0.col<-findInterval(theta0,theta)
    replace.col.with.nan<-col.with.nan
    for(indx in 1:nrow(col.with.nan)){
      if(col.with.nan[indx,2]<theta0.col){replace.col.with.nan[indx,2]<-ifelse(2*theta0.col-col.with.nan[indx,2]>37,(2*theta0.col-col.with.nan[indx,2])%%37,2*theta0.col-col.with.nan[indx,2])
      }else{ replace.col.with.nan[indx,2]<-ifelse(2*theta0.col-col.with.nan[indx,2]<1,(2*theta0.col-col.with.nan[indx,2])%%37,2*theta0.col-col.with.nan[indx,2])}
      S.w.theta[col.with.nan[indx,1],col.with.nan[indx,2]]<-S.w.theta[replace.col.with.nan[indx,1],replace.col.with.nan[indx,2]]
    }
  }
  return(S.w.theta)
}

Aux.Cov.Mat<-function(w,theta,lambda){
  mn<-length(w)*length(theta)
  CMat<-matrix(1,mn,mn)
  grid<-expand.grid(w,theta)
  auxmat1<-matrix(1,length(w),length(w))
  for(i in 1:length(w))for(j in i:length(w)) auxmat1[i,j]<-exp(-lambda[1]*abs(1/w[i]-1/w[j]))
  auxmat2<-matrix(1,length(theta),length(theta))
  for(i in 1:length(theta))for(j in i:length(theta)) auxmat2[i,j]<-exp(-lambda[2]*(1-cos(theta[i]-theta[j])))
  auxmat2[lower.tri(auxmat2)] <- t(auxmat2)[lower.tri(auxmat2)]
  auxmat1[lower.tri(auxmat1)] <- t(auxmat1)[lower.tri(auxmat1)]
  auxmat1eigen<-eigen(auxmat1)
  mat1<-auxmat1eigen$vectors %*% diag(sqrt(auxmat1eigen$values))%*%solve(auxmat1eigen$vectors)
  auxmat2eigen<-eigen(auxmat2)
  mat2<-auxmat2eigen$vectors %*% diag(sqrt(auxmat2eigen$values))%*%solve(auxmat2eigen$vectors)
  Sqr.CMat<-kronecker(mat2,mat1)
  #CMat<-kronecker(auxmat2,auxmat1)
  #for(i in 1:mn)for(j in i:mn) CMat[i,j]<-exp(-lambda[1]*abs(1/grid[i,1]-1/grid[j,1])-lambda[2]*(1-cos(grid[i,2]-grid[j,2])))
  #CMat[lower.tri(CMat)] <- t(CMat)[lower.tri(CMat)]
  return(Sqr.CMat)
}

Sim.DirSpec<-function(w,theta,hs,tp,theta0,smax=10,gam=NULL,U=NULL,n=1,plot=FALSE,sd.mat=diag(.1,nrow = 33*37)){
  # w and theta are vectors
  eval<-expand.grid(w,theta)
  Seval<-numeric(nrow(eval))
  for(i in 1:nrow(eval))Seval[i]<-JONSWAP(w=eval[i,1],hs,tp,gam)*G(theta=eval[i,2],w=eval[i,1],theta0,tp,smax,U)
  S.w.theta<-matrix(Seval,nrow=length(w),byrow=FALSE)
  S.w.theta<-check.S.nan(S.w.theta,theta0)
  NormS<-normalize(S.w.theta,normfx = TRUE,x = w,y = theta,D=2)
  if(n==1){
    Noise.Mat<-matrix(sd.mat%*%rnorm(length(S.w.theta),mean = 0),nrow=length(w),byrow=FALSE)
    Sim.S.w.theta<-S.w.theta+NormS/5*Noise.Mat
    Sim.S.w.theta[Sim.S.w.theta<0]<-0
    if(plot){
      X<-NULL
      Y<-NULL
      length.w<-length(w)
      for(i in 1:length.w)
      {
        X<-c(X,w[i]*cos(theta))
        Y<-c(Y,w[i]*sin(theta))
      }
      Xmat<-matrix(X,byrow=TRUE,nrow=length.w)
      Ymat<-matrix(Y,byrow=TRUE,nrow=length.w)
      par(mfrow=c(1,2))
      image2D(x=Xmat,y=Ymat,z= S.w.theta,colkey = list(side = 1, length = 0.5) ,theta = 0,
              axes=FALSE,main="Directional Spectrum",xlab=" ", ylab=" ")

      image2D(x=Xmat,y=Ymat,z= Sim.S.w.theta,colkey = list(side = 1, length = 0.5) ,theta = 0,
              axes=FALSE,main="Simulated Directional Spectrum",xlab=" ", ylab=" ")

    }
  }else{
    Sim.S.w.theta <-vector("list", n)
    for(i in 1:n){
      Noise.Mat<-matrix(sd.mat%*%rnorm(length(S.w.theta),mean = 0),nrow=length(w),byrow=FALSE)
      Sim.S.w.theta[[i]]<-S.w.theta+NormS/5*Noise.Mat
      Sim.S.w.theta[[i]][Sim.S.w.theta[[i]]<0]<-0
    }
    if(plot){
      X<-NULL
      Y<-NULL
      length.w<-length(w)
      for(i in 1:length.w)
      {
        X<-c(X,w[i]*cos(theta))
        Y<-c(Y,w[i]*sin(theta))
      }
      Xmat<-matrix(X,byrow=TRUE,nrow=length.w)
      Ymat<-matrix(Y,byrow=TRUE,nrow=length.w)
      par(mfrow=c(1,1))
      image2D(x=Xmat,y=Ymat,z= S.w.theta,colkey = list(side = 1, length = 0.5) ,theta = 0,
              axes=FALSE,main="Directional Spectrum",xlab=" ", ylab=" ")
    }
  }
  RESULTS<-list(w,theta,S.w.theta,Sim.S.w.theta)
  names(RESULTS)<-c("w","theta","S.w.theta","Sim.S.w.theta")
  return(RESULTS)
}

Sim.DirSpec2<-function(w,theta,hs,hs2=hs,tp,tp2,theta0,theta02,gam=NULL,gam2=gam,smax=10,smax2=smax,U=NULL,U2=U,n=1,plot=FALSE,sd.mat=diag(.1,nrow = 33*37)){
  #Note tp=tp2 has not physical interpretation
  # w and theta are vectors
  eval<-expand.grid(w,theta)
  Seval<-numeric(nrow(eval))
  Seval2<-numeric(nrow(eval))
  for(i in 1:nrow(eval)){
    Seval[i]<-JONSWAP(w=eval[i,1],hs,tp,gam)*G(theta=eval[i,2],w=eval[i,1],theta0,tp,smax,U)
    Seval2[i]<-JONSWAP(w=eval[i,1],hs2,tp2,gam2)*G(theta=eval[i,2],w=eval[i,1],theta02,tp2,smax2,U2)
  }
  S1.w.theta<-matrix(Seval,nrow=length(w),byrow=FALSE)
  S1.w.theta<-check.S.nan(S1.w.theta,theta0)
  S2.w.theta<-matrix(Seval2,nrow=length(w),byrow=FALSE)
  S2.w.theta<-check.S.nan(S2.w.theta,theta02)
  S.w.theta<-(S1.w.theta/sum(S1.w.theta)+S2.w.theta/sum(S2.w.theta))*(sum(S1.w.theta)+sum(S2.w.theta))/2
  if(n==1){
    Noise.Mat<-matrix(sd.mat%*%rnorm(length(S.w.theta),mean = 0),nrow=length(w),byrow=FALSE)
    Sim.S.w.theta<-S.w.theta+Noise.Mat
    Sim.S.w.theta[Sim.S.w.theta<0]<-0
    if(plot){
      X<-NULL
      Y<-NULL
      length.w<-length(w)
      for(i in 1:length.w)
      {
        X<-c(X,w[i]*cos(theta))
        Y<-c(Y,w[i]*sin(theta))
      }
      Xmat<-matrix(X,byrow=TRUE,nrow=length.w)
      Ymat<-matrix(Y,byrow=TRUE,nrow=length.w)
      par(mfrow=c(1,2))
      image2D(x=Xmat,y=Ymat,z= S.w.theta,colkey = list(side = 1, length = 0.5) ,theta = 0,
              axes=FALSE,main="Directional Spectrum",xlab=" ", ylab=" ")

      image2D(x=Xmat,y=Ymat,z= Sim.S.w.theta,colkey = list(side = 1, length = 0.5) ,theta = 0,
              axes=FALSE,main="Simulated Directional Spectrum",xlab=" ", ylab=" ")

    }
  }else{
    Sim.S.w.theta <-vector("list", n)
    for(i in 1:n){
      Noise.Mat<-matrix(sd.mat%*%rnorm(length(S.w.theta),mean = 0),nrow=length(w),byrow=FALSE)
      Sim.S.w.theta[[i]]<-S.w.theta+Noise.Mat
      Sim.S.w.theta[[i]][Sim.S.w.theta[[i]]<0]<-0
    }
    if(plot){
      X<-NULL
      Y<-NULL
      length.w<-length(w)
      for(i in 1:length.w)
      {
        X<-c(X,w[i]*cos(theta))
        Y<-c(Y,w[i]*sin(theta))
      }
      Xmat<-matrix(X,byrow=TRUE,nrow=length.w)
      Ymat<-matrix(Y,byrow=TRUE,nrow=length.w)
      par(mfrow=c(1,1))
      image2D(x=Xmat,y=Ymat,z= S.w.theta,colkey = list(side = 1, length = 0.5) ,theta = 0,
              axes=FALSE,main="Directional Spectrum",xlab=" ", ylab=" ")
    }
  }
  RESULTS<-list(w,theta,S.w.theta,Sim.S.w.theta)
  names(RESULTS)<-c("w","theta","S.w.theta","Sim.S.w.theta")
  return(RESULTS)
}

Sim.DirSpeck<-function(w,theta,SpecList,n=1,plot=FALSE,sd.mat=diag(.1,nrow = 33*37)){
  eval<-expand.grid(w,theta)
  #SpecList should be normalized
  S.w.theta<-Reduce("+",SpecList)
  NormS<-normalize(S.w.theta,normfx = TRUE,x = w,y = theta,D=2)
  if(n==1){
    Noise.Mat<-matrix(sd.mat%*%rnorm(length(S.w.theta),mean = 0),nrow=length(w),byrow=FALSE)
    Sim.S.w.theta<-S.w.theta+NormS/5*Noise.Mat
    Sim.S.w.theta[Sim.S.w.theta<0]<-0
    if(plot){
      X<-NULL
      Y<-NULL
      length.w<-length(w)
      for(i in 1:length.w)
      {
        X<-c(X,w[i]*cos(theta))
        Y<-c(Y,w[i]*sin(theta))
      }
      Xmat<-matrix(X,byrow=TRUE,nrow=length.w)
      Ymat<-matrix(Y,byrow=TRUE,nrow=length.w)
      par(mfrow=c(1,2))
      image2D(x=Xmat,y=Ymat,z= S.w.theta,colkey = list(side = 1, length = 0.5) ,theta = 0,
              axes=FALSE,main="Directional Spectrum",xlab=" ", ylab=" ")

      image2D(x=Xmat,y=Ymat,z= Sim.S.w.theta,colkey = list(side = 1, length = 0.5) ,theta = 0,
              axes=FALSE,main="Simulated Directional Spectrum",xlab=" ", ylab=" ")

    }
  }else{
    Sim.S.w.theta <-vector("list", n)
    for(i in 1:n){
      Noise.Mat<-matrix(sd.mat%*%rnorm(length(S.w.theta),mean = 0),nrow=length(w),byrow=FALSE)
      Sim.S.w.theta[[i]]<-S.w.theta+NormS/5*Noise.Mat
      Sim.S.w.theta[[i]][Sim.S.w.theta[[i]]<0]<-0
    }
    if(plot){
      X<-NULL
      Y<-NULL
      length.w<-length(w)
      for(i in 1:length.w)
      {
        X<-c(X,w[i]*cos(theta))
        Y<-c(Y,w[i]*sin(theta))
      }
      Xmat<-matrix(X,byrow=TRUE,nrow=length.w)
      Ymat<-matrix(Y,byrow=TRUE,nrow=length.w)
      par(mfrow=c(1,1))
      image2D(x=Xmat,y=Ymat,z= S.w.theta,colkey = list(side = 1, length = 0.5) ,theta = 0,
              axes=FALSE,main="Directional Spectrum",xlab=" ", ylab=" ")
    }
  }
  RESULTS<-list(w,theta,S.w.theta,Sim.S.w.theta)
  names(RESULTS)<-c("w","theta","S.w.theta","Sim.S.w.theta")
  return(RESULTS)
}

Partitioning<-function(w,theta,Spec,tol=.01,min.area=.1){
  X<-normalize(x=w,fx=Spec,D = 2,y = theta)
  PDS1<-watershed(X,tolerance = tol,ext = 1)
  Aux<-NULL
  for(i in 0:max(PDS1)){
    if(sum(X[PDS1==i])>min.area){
      indexes.w.theta<-which(PDS1==i,arr.ind = TRUE)
      max.ind<-indexes.w.theta[which.max(X[indexes.w.theta]),]
      Aux<-c(Aux,mean(w[indexes.w.theta[,1]]),mean(theta[indexes.w.theta[,2]]),w[max.ind[1]],theta[max.ind[2]])
    }
  }
  PartSum<-matrix(Aux,ncol=4,byrow = TRUE)
  colnames(PartSum)<-c("w.mean","theta.mean","w.p","theta.p")
  return(PartSum)
}

polarplotg<-function(theta,gw){
  length.t<-length(theta)
  gw[c(1,length.t)]<-(gw[1]+gw[length.t])/2
  d <- structure(list(Theta = theta, Energy = gw)
                 , .Names = c("Theta", "Energy"),
                 row.names = c(NA, length(theta)), class = "data.frame")
  gplot<-ggplot(d, aes(x = Theta, y = Energy)) +
    ggtitle("Directional Distribution") +
    theme(plot.title = element_text(hjust = 0.5,size = 18, vjust=10))+
    geom_path(lwd=1.5,col=4)+
    coord_polar(theta = "x", start = -pi/2 ,direction = -1) +
    scale_x_continuous(breaks = seq(0, 360, 45))
  print(gplot)
}

WrapBP<-function(theta,G,colorP="purple", index=NULL){
  aux.fbplot<-fbplot(x=theta,fit=G,plot = FALSE)
  Median<-G[,aux.fbplot$medcurve]
  aux.band<-order(aux.fbplot$depth,decreasing = TRUE)[1:(ceiling(ncol(G)/2))]
  bandlow<-apply(G[,aux.band],MARGIN = 1,min)
  bandtop<-apply(G[,aux.band],MARGIN = 1,max)
  length.t<-length(theta)
  d <- structure(list(Angle = theta, Energy = Median,Energylow=bandlow,Energytop=bandtop)
                 , .Names = c("Angle", "Energy","Energylow","Energytop"),
                 row.names = c(NA, 37L), class = "data.frame")
  gplot<-ggplot(d, aes(x = Angle, y = Energy)) +
    coord_polar(theta = "x", start = -pi/2,direction = -1)+
    scale_x_continuous(breaks =seq(0,7*pi/4,pi/4), labels=seq(0, 315, 45))+
    geom_ribbon(data = d,aes(x=Angle,ymin = Energylow,ymax=Energytop),
                col=colorP,fill=colorP)+geom_path(lwd=1.5)+
    ggtitle(paste("Cluster ",index))+theme(plot.title = element_text(size=32, hjust=.5,face="bold")) +
    theme(axis.title = element_text(size=20))+theme(axis.text = element_text(size=18))

  print(gplot)
}


multiplot <- function(..., plotlist=NULL, file, cols=1, layout=NULL) {
  library(grid)

  # Make a list from the ... arguments and plotlist
  plots <- c(list(...), plotlist)

  numPlots = length(plots)

  # If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                     ncol = cols, nrow = ceiling(numPlots/cols))
  }

  if (numPlots==1) {
    print(plots[[1]])

  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))

    # Make each plot, in the correct location
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))

      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                      layout.pos.col = matchidx$col))
    }
  }
}


#Similarity between two clusters
Sim<-function(A,B)
{
  m<-ncol(A)
  n<-ncol(B)
  S<-matrix(0,nrow=m,ncol=n)
  mS<-c()
  for(i in 1:m){
    for(j in 1:n)S[i,j]<-2*(length(intersect(A[,i],B[,j]))/(sum(is.na(A[,i])==FALSE)+length(B[,j])))
    mS[i]<-max(S[i,])
  }
  return(sum(mS)/n)
}

#HMClust

cutk<-function(Clust,kg=NA,alpha=NA)
{
  #Clust output from HSM
  if(length(na.omit(kg))>0){
    initial_k<-length(Clust$min.value)+1
    GG<-Clust$Groups[[initial_k-kg]]
    return(GG)
  }
  if(length(na.omit(alpha))>0){
    initial_k<-length(Clust$min.value)+1
    kg<-initial_k-(which(Clust$min.value>alpha)[1]-1)
    if(length(na.omit(kg))>0){
      GG<-Clust$Groups[[initial_k-kg]]
      return(GG)
    }
    else{print("One Cluster")}
  }
  if(length(na.omit(alpha))==0 & length(na.omit(kg))==0) print("Give me a value of alpha or kg")
}

TVD<-function(w,f1,f2){
  #Compute TV distance using the trapezoidal rule
  if(length(w)!=length(f1)|length(w)!=length(f2))stop("w, f1 and f2 must have the same length")
  n<-length(w)
  int<-(w[2:n]-w[1:(n-1)])%*%(pmin(f1,f2)[2:n]+pmin(f1,f2)[1:(n-1)])/2
  return(as.double(1-int))
}

HSM<-function(X,S=NULL,w=NULL,freq=1,Merger=1,par.spectrum=c(100,1/(2*dt),512),parallel=FALSE){
  ##Compute the HSM algorithm using the TVD
  dt<-1/freq
  if(is.null(S)){
    #Time series as input
    #Initial parameters
    n<-length(X[,1])
    k<-length(X[1,])
    #Time<-n*dt
    np<-length(par.spectrum)
    #if(np==0){a<-100; wn<-1/(2*dt);length.w<-512}
    if(np==1){a<-par.spectrum[1]; wn<-1/(2*dt); length.w<-512}
    if(np==2){a<-par.spectrum[1]; wn<-par.spectrum[2]; length.w<-512}
    if(np==3){a<-par.spectrum[1]; wn<-par.spectrum[2]; length.w<-par.spectrum[3]}
    w<-seq(10^(-5),wn,length.out=length.w)
    #Initial normalized estimated Spectra and Dissimilarity Matrix
    if(parallel){
      library("doParallel")
      cl<-min(k,detectCores()-1)
      registerDoParallel(cl)
      Spec<-foreach(i=1:(k-1),.combine = cbind) %dopar%
      {
        Aux_Spec<-spec.parzen(X[,i],a=a,dt=dt,wn=wn,nn=length.w)
        Aux_Spec[,2]/((w[2:length.w]-w[1:(length.w-1)])%*%(Aux_Spec[2:length.w,2]+Aux_Spec[1:(length.w-1),2])/2)
      }
      Aux_Spec<-spec.parzen(X[,k],a=a,dt=dt,wn=wn,nn=length.w)
      Spec<-cbind(Spec,Aux_Spec[,2]/((w[2:length.w]-w[1:(length.w-1)])%*%(Aux_Spec[2:length.w,2]+Aux_Spec[1:(length.w-1),2])/2))

    }else{
      Spec<-matrix(NA,nrow=length.w,ncol=k)
      for(i in 1:k){
        Aux_Spec<-spec.parzen(X[,i],a=a,dt=dt,wn=wn,nn=length.w)
        w<-Aux_Spec[,1]
        Spec[,i]<-Aux_Spec[,2]/((w[2:length.w]-w[1:(length.w-1)])%*%(Aux_Spec[2:length.w,2]+Aux_Spec[1:(length.w-1),2])/2)
      }}
  }else{
    #Spectrum as input
    Spec<-S
    k<-ncol(S)
    if(is.null(w)){
      np<-length(par.spectrum)
      if(np==1){a<-par.spectrum[1]; wn<-1/(2*dt); length.w<-512}
      if(np==2){a<-par.spectrum[1]; wn<-par.spectrum[2]; length.w<-512}
      if(np==3){a<-par.spectrum[1]; wn<-par.spectrum[2]; length.w<-par.spectrum[3]}
      w<-seq(10^(-5),wn,length.out=nn)
    }
    length.w<-length(w)
    for(i in 1:k){
      Aux_Spec<-Spec[,i]
      Spec[,i]<-Aux_Spec/((w[2:length.w]-w[1:(length.w-1)])%*%(Aux_Spec[2:length.w]+Aux_Spec[1:(length.w-1)])/2)
    }
    if(Merger==1){
      print("Merger has been assigned to 2")
      Merger<-2
    }
  }
  if(parallel){
    library("doParallel")
    cl<-min(k,detectCores()-1)
    registerDoParallel(cl)
    MatDiss<-foreach(i=1:k,.combine = rbind) %dopar%
    {
      Aux.MatDiss<-rep(NA,k)
      for(j in i:k) Aux.MatDiss[j]<-TVD(w,Spec[,i],Spec[,j])
      Aux.MatDiss
    }
  }else{
    MatDiss<-matrix(NA,k,k)
    for(i in 1:k)for(j in i:k) MatDiss[i,j]<-TVD(w,Spec[,i],Spec[,j])
  }
  diag(MatDiss)<-0
  MatDiss[lower.tri(MatDiss)]<-t(MatDiss)[lower.tri(MatDiss)]
  #Dinamic Diss Mat
  MD_Change<-MatDiss# initial MD
  diag(MD_Change)<-rep(Inf,k)
  #Dinamic estimated spectra
  Spec_Change<-Spec
  # initial groups
  g<-as.list(1:k)
  #Save variables
  min.value<-numeric(k-1)
  groups<-list()
  #Version 1
  if(Merger==1){
    Xlist<-list()
    for(j in 1:k)Xlist[[j]]<-X[,j]#initial series
    for(ite in 1:(k-1)){
      #############
      # Identify the closest clusters and the minimun value
      min.value[ite]<-min(MD_Change) ##
      aux1<-which(MD_Change==min(MD_Change),arr.ind = TRUE)[1,]
      g<-c(g[-aux1],list(unlist(g[aux1])))# new groups
      groups[[ite]]<-g ##
      #############
      # Merge Time Series
      Xlist<-c(Xlist[-aux1],list(unlist(Xlist[sort(aux1)])))
      Spec_Change<-Spec_Change[,-aux1]
      Spec_Xnew<-spec.parzen(Xlist[[k-ite]],a=a,dt=dt,wn=wn,nn=length.w)
      Spec_Change<-cbind(Spec_Change,Spec_Xnew[,2]/((w[2:length.w]-w[1:(length.w-1)])%*%(Spec_Xnew[2:length.w,2]+Spec_Xnew[1:(length.w-1),2])/2))
      #############
      #Compute new TVD
      MD_Change<-MD_Change[-aux1,-aux1]
      if(ite<(k-1)){
        aux2<-numeric(k-ite-1)
        for(i in 1:(k-ite-1)){
          aux2[i]<-TVD(w,Spec_Change[,i],Spec_Change[,k-ite])
        }
        MD_Change<-rbind(MD_Change,aux2)
        MD_Change<-cbind(MD_Change,c(aux2,Inf))# new MD
      }else{MD_Change<-0}
    }
  }
  #Version 2
  if(Merger==2){
    for(ite in 1:(k-1)){
      #############
      # Identify the closest clusters and the minimun value
      min.value[ite]<-min(MD_Change)
      aux1<-which(MD_Change==min(MD_Change),arr.ind = TRUE)[1,]
      g<-c(g[-aux1],list(unlist(g[aux1])))# new groups
      groups[[ite]]<-g ##
      #############
      # Spectral Merge
      Spec_Change<-Spec_Change[,-aux1]
      Spec_Change<-cbind(Spec_Change,rowMeans(Spec[,g[[k-ite]]]))
      #############
      #Compute new TVD
      MD_Change<-MD_Change[-aux1,-aux1]
      if(ite<(k-1)){
        aux2<-numeric(k-ite-1)
        for(i in 1:(k-ite-1)){
          aux2[i]<-TVD(w,Spec_Change[,i],Spec_Change[,k-ite])
        }
        MD_Change<-rbind(MD_Change,aux2)
        MD_Change<-cbind(MD_Change,c(aux2,Inf))# new MD
      }else{MD_Change<-0}

    }
  }
  out <- list(MatDiss,min.value, groups)
  names(out) <- c("Diss.Matrix","min.value", "Groups")
  out
}

